;;; guile-irrlicht --- FFI bindings for Irrlicht Engine
;;; Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>
;;;
;;; This file is part of guile-irrlicht.
;;;
;;; Guile-irrlicht is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; Guile-irrlicht is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU Lesser General Public
;;; License along with guile-irrlicht.  If not, see
;;; <http://www.gnu.org/licenses/>.


(define-module (irrlicht device)
  #:use-module (oop goops)
  #:use-module (irrlicht base)
  #:use-module (irrlicht foreign)
  #:use-module (irrlicht io)
  #:use-module (irrlicht irr)
  #:use-module (irrlicht gui)
  #:use-module (irrlicht scene)
  #:use-module (irrlicht video))


;; IrrlichtDevice
(define-class <irrlicht-device> (<reference-counted>)
  (irr-class #:init-value "IrrlichtDevice"))

(define* (create-device #:key
                        (device-type 'software)
                        (window-size '(640 480))
                        (bits 16)
                        (fullscreen #f)
                        (stencilbuffer #f)
                        (vsync #f)
                        (receiver (make <event-receiver>)))
  (if (not (is-a? receiver <event-receiver>))
      (error
       "In procedure create-device: Wrong type argument (expecting <event-receiver>):"
       receiver))

  (let* ((createDevice (get-irrlicht-proc "createDevice"))
         (device-pointer (createDevice device-type window-size bits fullscreen stencilbuffer
                                       vsync receiver)))
    (cond ((null-pointer? device-pointer)
           (error "In procedure create-device: Device cannot be created"))
          (else
           (make <irrlicht-device> #:irr-pointer device-pointer)))))

(define-method (get-cursor-control (device <irrlicht-device>))
  (let ((getCursorControl (get-irrlicht-proc "getCursorControl" device)))
    (make <cursor-control>
      #:irr-pointer (getCursorControl device))))

(define-method (get-file-system (device <irrlicht-device>))
  (let ((getFileSystem (get-irrlicht-proc "getFileSystem" device)))
    (make <file-system>
      #:irr-pointer (getFileSystem device))))

(define-method (get-gui-environment (device <irrlicht-device>))
  (let ((getGUIEnvironment (get-irrlicht-proc "getGUIEnvironment" device)))
    (make <gui-environment>
      #:irr-pointer (getGUIEnvironment device))))

(define-method (get-scene-manager (device <irrlicht-device>))
  (make <scene-manager>
    #:irr-pointer ((get-irrlicht-proc "getSceneManager" device) device)))

(define-method (get-video-driver (device <irrlicht-device>))
  (make <video-driver>
    #:irr-pointer ((get-irrlicht-proc "getVideoDriver" device) device)))

(define-method (is-window-active? (device <irrlicht-device>))
  (let ((isWindowActive (get-irrlicht-proc "isWindowActive" device)))
    (isWindowActive device)))

(define-method (run (device <irrlicht-device>))
  ((get-irrlicht-proc "run" device) device))

(define-method (set-window-caption! (device <irrlicht-device>) text)
  ((get-irrlicht-proc "setWindowCaption" device)
   device text))

(define-method (yield-device (device <irrlicht-device>))
  (let ((yield (get-irrlicht-proc "yield" device)))
    (yield device)))

(export create-device get-cursor-control get-file-system get-gui-environment get-scene-manager
        get-video-driver is-window-active? run set-window-caption! yield-device)
