;;; guile-irrlicht --- FFI bindings for Irrlicht Engine
;;; Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>
;;;
;;; This file is part of guile-irrlicht.
;;;
;;; Guile-irrlicht is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; Guile-irrlicht is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU Lesser General Public
;;; License along with guile-irrlicht.  If not, see
;;; <http://www.gnu.org/licenses/>.


(define-module (irrlicht gui)
  #:use-module (oop goops)
  #:use-module (ice-9 optargs)
  #:use-module (irrlicht base)
  #:use-module (irrlicht foreign)
  #:use-module (irrlicht io)
  #:use-module (irrlicht irr))


;; ICursorControl
(define-class <cursor-control> (<reference-counted>)
  (irr-class #:init-value "ICursorControl"))

(define-method (set-visible! (cursor-control <cursor-control>) visible)
  (let ((setVisible (get-irrlicht-proc "setVisible" cursor-control)))
    (setVisible cursor-control visible)))

(export <cursor-control> set-visible!)


;; IGUIElement
(define-class <gui-element> (<attribute-exchanging-object> <event-receiver>)
  (irr-class #:init-value "IGUIElement"))

(export <gui-element>)


;; IGUIEnvironment
(define-class <gui-environment> (<reference-counted>)
  (irr-class #:init-value "IGUIEnvironment"))

(define-method (add-static-text! (gui-environment <gui-environment>) text rectangle . rest)
  (let-keywords rest #f
        ((border #f)
         (word-wrap #t)
         (parent (make <gui-element>))
         (id -1)
         (fill-background #f))
    (make <gui-static-text>
      #:irr-pointer
      ((get-irrlicht-proc "addStaticText" gui-environment parent)
       gui-environment
       text
       rectangle
       border
       word-wrap
       parent
       id
       fill-background))))

(define-method (draw-all (gui-environment <gui-environment>))
  ((get-irrlicht-proc "drawAll" gui-environment)
   gui-environment))

(export <gui-environment> add-static-text! draw-all)


;; IGUIStaticText
(define-class <gui-static-text> (<gui-element>)
  (irr-class #:init-value "IGUIStaticText"))

(export <gui-static-text>)
