/* guile-irrlicht --- GNU Guile bindings for Irrlicht Engine

   Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>

   This file is part of guile-irrlicht.

   guile-irrlicht is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 3 of the
   License, or (at your option) any later version.

   guile-irrlicht is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with guile-irrlicht. If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <irrlicht/irrlicht.h>
#include <libguile.h>
#include "cursor-control.h"
#include "gsubr.h"
#include "position2d.h"
#include "wrapped.h"

using namespace irr;

SCM
ICursorControl_getPosition (SCM cursor_control)
{
  return scm_from_position2d_s32
    (((gui::ICursorControl*)scm_to_irr_pointer (cursor_control))->getPosition ());
}

SCM
ICursorControl_setPosition (SCM cursor_control,
                            SCM position)
{
  ((gui::ICursorControl*)scm_to_irr_pointer (cursor_control))->
    setPosition (scm_to_position2d_s32 (position));
  return SCM_UNSPECIFIED;
}

SCM
ICursorControl_setVisible (SCM cursor_control,
                           SCM visible)
{
  ((gui::ICursorControl*)scm_to_irr_pointer (cursor_control))->
    setVisible (scm_to_bool (visible));
  return SCM_UNSPECIFIED;
}

void
init_cursor_control (void)
{
  DEFINE_GSUBR ("ICursorControl_getPosition", 1, 0, 0, ICursorControl_getPosition);
  DEFINE_GSUBR ("ICursorControl_setPosition", 2, 0, 0, ICursorControl_setPosition);
  DEFINE_GSUBR ("ICursorControl_setVisible", 2, 0, 0, ICursorControl_setVisible);
}
